classdef baconDecomp 
    % Written by Dr. Ralf ELsas, LMU SOM Munich
    % - Version of 12/20/2024
    % - Please cite when you use the program and write me an Email for all
    %   errors or improvements
    properties  
        data  
        timeVar  
        idVar  
        treatVar  
        outcomeVar  
        treatmentTimes  
    end  

    methods  
        function obj = baconDecomp(data, timeVar, idVar, treatVar, outcomeVar)  
            % Constructor to initialize the class  
            obj.data = data;  
            obj.timeVar = timeVar;  
            obj.idVar = idVar;  
            obj.treatVar = treatVar;  
            obj.outcomeVar = outcomeVar;  
            
            werNaN = find(isnan(obj.data.treatVar));
            obj.data.treatVar(werNaN) =0;

            % Get unique treatment times (excluding never-treated group)  
            obj.treatmentTimes = unique(data.(treatVar));  
            obj.treatmentTimes = obj.treatmentTimes(obj.treatmentTimes > 0);  
        end  

        function [results, results_aggregate] = decompose(obj)  
            % Initialize arrays to store results  
            n_comparisons = nchoosek(length(obj.treatmentTimes), 2) * 2 + ...  
                            length(obj.treatmentTimes);  % Earlier vs Later + Treated vs Never  
            type = cell(n_comparisons, 1);  
            group1 = zeros(n_comparisons, 1);  
            group2 = zeros(n_comparisons, 1);  
            weight = zeros(n_comparisons, 1);  
            estimate = zeros(n_comparisons, 1);  

            % Counter for storing results  
            idx = 1;  
              % Compute total variance of treatment indicator (V^D)  
            VD = var(obj.data.(obj.treatVar) >0 );  

            % Treated vs Never Treated comparisons  
            for i = 1:length(obj.treatmentTimes)  
                time = obj.treatmentTimes(i);  
                [est, ~] = obj.computeTreatedVsNever(time);  
                weight(idx) = obj.computeTreatedVsNeverWeight(time, VD);  

                type{idx} = 'Treated vs Never Treated';  
                group1(idx) = time;  
                group2(idx) = 0;  
                estimate(idx) = est;  
                idx = idx + 1;  
            end  

            % Earlier vs Later Treated comparisons  
            for i = 1:length(obj.treatmentTimes)  
                for j = i+1:length(obj.treatmentTimes)  
                    time1 = obj.treatmentTimes(i);  
                    time2 = obj.treatmentTimes(j);  
                    [est, ~] = obj.computeTwoByTwo(time1, time2);  
                    weight(idx) = obj.computeEarlierVsLaterWeight(time1, time2, VD);  

                    type{idx} = 'Treated earlier vs later';  
                    group1(idx) = time1;  
                    group2(idx) = time2;  
                    estimate(idx) = est;  
                    idx = idx + 1;  
                end  
            end  

            % Later vs Earlier comparisons  
            for i = 1:length(obj.treatmentTimes)  
                for j = 1:i-1  
                    time1 = obj.treatmentTimes(i);  
                    time2 = obj.treatmentTimes(j);  
                    [est, ~] = obj.computeTwoByTwo(time1, time2);  
                    weight(idx) = obj.computeLaterVsEarlierWeight(time1, time2, VD);  

                    type{idx} = 'Treated later vs earlier';  
                    group1(idx) = time1;  
                    group2(idx) = time2;  
                    estimate(idx) = est;  
                    idx = idx + 1;  
                end  
            end  

            % Normalize weights to sum to 1  
            weight = weight / sum(weight);  

            % Create results table  
            results = table(type, group1, group2, weight, estimate); 
            g_sum = groupsummary(results,"type",["sum"],["weight"]);
            g_sum2 = groupsummary(results,"type",@(x,y) (x'*y)./sum(x),{"weight","estimate"});
           
            results_aggregate = g_sum(:,["type","sum_weight"]);
            results_aggregate.estimate = g_sum2.fun1_weight_estimate;
            results_aggregate.Properties.VariableNames = ["Type","Weight","Estimate"];
            
        end  

        function [estimate, weight] = computeTreatedVsNever(obj, treatTime)  
            % Compute the 2x2 DD estimate for treated vs never-treated  
            treated = obj.data.(obj.treatVar) == treatTime;  
            control = obj.data.(obj.treatVar) == 0;  
             % Pre-treatment period  
            prePeriod = obj.data.(obj.timeVar) < treatTime;  

            % Post-treatment period  
            postPeriod = obj.data.(obj.timeVar) >= treatTime;  

            % Count units in each group  
            nt = sum(treated);  
            nc = sum(control);  

            % Compute means  
            y = obj.data.(obj.outcomeVar);  
            treated_pre = mean(y(treated & prePeriod), 'omitnan');  
            treated_post = mean(y(treated & postPeriod), 'omitnan');  
            control_pre = mean(y(control & prePeriod), 'omitnan');  
            control_post = mean(y(control & postPeriod), 'omitnan');  

            % DiD estimate  
            estimate = (treated_post - treated_pre) - (control_post - control_pre);  

            % Weight calculation (placeholder, actual weight is computed separately)  
            weight = 0;  
        end  

        function [estimate, weight] = computeTwoByTwo(obj, time1, time2)  
            % Compute the 2x2 DD estimate for earlier vs later treated  
            group1 = obj.data.(obj.treatVar) == time1;  
            group2 = obj.data.(obj.treatVar) == time2;  

            % Pre-treatment period (before first treatment)  
            if time1<time2
                prePeriod = obj.data.(obj.timeVar) < time1;  
            else
                prePeriod = obj.data.(obj.timeVar) >= time1;
            end
            % Between-treatment period  
            if time1<time2
                betweenPeriod = (obj.data.(obj.timeVar) >= time1) & ...  
                            (obj.data.(obj.timeVar) < time2);  
            else
                betweenPeriod = (obj.data.(obj.timeVar) >= time2) & ...  
                            (obj.data.(obj.timeVar) < time1);  
            end
            % Count units in each group  
            n1 = sum(group1);  
            n2 = sum(group2);  

            % Compute means  
            y = obj.data.(obj.outcomeVar);  
                g1_pre = mean(y(group1 & prePeriod), 'omitnan');  
                g1_between = mean(y(group1 & betweenPeriod), 'omitnan');  
                g2_pre = mean(y(group2 & prePeriod), 'omitnan');  
                g2_between = mean(y(group2 & betweenPeriod), 'omitnan');  
                % DiD estimate  
            if time1<time2
                estimate = (g1_between - g1_pre) - (g2_between - g2_pre);  
            else
                estimate = (g1_pre-g1_between) - (g2_pre - g2_between );  
            end
            % Weight calculation (placeholder, actual weight is computed separately)  
            weight = 0;  
        end  

        function [weight] = computeTreatedVsNeverWeight(obj, k, VD)  
            % Compute weight for treated vs never-treated comparison (Equation 10e)  
            treated = obj.data.(obj.treatVar) == k;  
            never_treated = obj.data.(obj.treatVar) == 0;  
            isSample = treated|never_treated;
            nk = sum(treated);  
            nu = sum(never_treated);  
            
            % nk = nk/(nk+nu);
            % nu = nu/(nk+nu);
            % Compute D_k (treatment indicator proportion)  
            post_k = obj.data.(obj.timeVar)(isSample) >= k;  
            Dk = mean(post_k);  

            % Compute n_ku  
            % nku = mean(treated | never_treated);  
            nku = nk./(nk+ nu);
            % Compute weight according to equation 10e  
            weight = ((nk + nu)^2 * nku * (1 - nku) * Dk * (1 - Dk)) / VD;  
        end  

        function [weight] = computeEarlierVsLaterWeight(obj, k, l, VD)  
            % Compute weight for earlier vs later treated comparison (Equation 10f)  
            treated_k = obj.data.(obj.treatVar) == k;  
            treated_l = obj.data.(obj.treatVar) == l;  

            nk = sum(treated_k);  
            nl = sum(treated_l);  
            
            
            % Compute D_k and D_l  
            post_k = obj.data.(obj.timeVar) >= k;  
            post_l = obj.data.(obj.timeVar) >= l;  
            Dk = mean(post_k);  
            Dl = mean(post_l);  

            % Compute n_kl  
            % nkl = mean(treated_k | treated_l);  
            nkl = nk./(nk+nl);
            % Compute weight according to equation 10f  
            weight = (((nk + nl) * (1 - Dl))^2 * nkl * (1 - nkl) * ...  
                     ((Dk - Dl) / (1 - Dl)) * ((1 - Dk) / (1 - Dl))) / VD;  
        end  

        function [weight] = computeLaterVsEarlierWeight(obj, k, l, VD)  
            % Compute weight for later vs earlier treated comparison (Equation 10g)  
            treated_k = obj.data.(obj.treatVar) == k;  
            treated_l = obj.data.(obj.treatVar) == l;  
            nk = sum(treated_k);  
            nl = sum(treated_l);  
            
            
            % Compute D_k and D_l  
            post_k = obj.data.(obj.timeVar) >= k;  
            post_l = obj.data.(obj.timeVar) >= l;
            Dk = mean(post_k);  
            Dl = mean(post_l);  

            % Compute n_kl  
             nkl = nk./(nk+nl);
            % Compute weight according to equation 10g  
            weight = (((nk + nl) * Dl)^2 * nkl * (1 - nkl) * ...  
                     (Dk / Dl) * ((Dl - Dk) / Dl)) / VD;  
        end  
    end  
end  