function woolDID(data)
    % Function to estimate dynamic treatment effects using pooled OLS as per Wooldridge (2021) Section 5.2
    % 
    % Input: data - A table with columns id, time, y (outcome), and
    %               treatPost, at least in this order!!
    %             - treatPost is an indicator variable taking on value 1 at a time period with treatment, and zero else. 
    %
    % ----------------------------------------------
    % Written by Dr. Ralf ELsas, LMU SOM Munich
    % - Version of 12/20/2024
    % - Please cite when you use the program and write me an Email for all
    %   errors or improvements
    % ----------------------------------------------
    vnames = ["id","time","y","treatPost"];
    if ~all(ismember(vnames,data.Properties.VariableNames))
        data.Properties.VariableNames = vnames;
    end
    
    %% Prepare the data
    % Ensure data is sorted by firm_id and year
    data = sortrows(data, {'id', 'time'});

    % Extract unique years and cohorts
    years = unique(data.time);
    cohorts = unique(data.time(find(data.treatPost(~(isnan(data.treatPost))))));
    data.treatment_year = data.time.*data.treatPost;
    data.treatment_year(data.treatment_year==0)=NaN;
    firstTreatment = groupsummary(data, "id", "min", "treatment_year", 'IncludeEmptyGroups', true);
    [wer,wo] = ismember(data.id,firstTreatment.id);
    data.treatment_year = zeros(height(data),1);
    data.treatment_year = firstTreatment.min_treatment_year(wo(wer));
    data.treatment_year(isnan(data.treatment_year))=0;
    
  
    %% Prepare variables for pooled OLS
    % Dependent variable
    Y = data.y;

    % Independent variables: constant term, cohort-specific treatment indicators, and year dummies
    X = ones(height(data), 1); % Constant term (alpha)
    variable_names = {'Constant'}; % Names for coefficients

    % Add year dummies (f_{qt})
    for t = 2:length(years) % Start from the second year to exclude the reference year
        year = years(t);
        year_dummy = (data.time == year);
        X = [X, year_dummy];
        variable_names = [variable_names, {['Year_' num2str(year)]}];
    end

    % Add cohort dummies (d_i)
    for c = 1:length(cohorts)
        cohort = cohorts(c);
        cohort_dummy = (data.treatment_year == cohort);
        X = [X, cohort_dummy];
        variable_names = [variable_names, {['Cohort_' num2str(cohort)]}];
    end

    % Add interactions between cohort dummies (d_i) and year dummies (f_{qt})
    for c = 1:length(cohorts)
        cohort = cohorts(c);
        for t = cohort:max(years) % Only consider years >= cohort
            year = t;

            % Only include interactions for treated years within the cohort
            if any((data.treatment_year == cohort) & (data.time == year))
                interaction_term = (data.treatment_year == cohort) .* (data.time == year);
                X = [X, interaction_term];
                variable_names = [variable_names, {['ATT_C' num2str(cohort) '_Y' num2str(year)]}];
            end
        end
    end

    %% Perform pooled OLS regression
    % Remove missing data rows
    valid_rows = ~any(isnan(X), 2) & ~isnan(Y);
    X_valid = X(valid_rows, :);
    Y_valid = Y(valid_rows);

    % Estimate coefficients using OLS
    beta = (X_valid' * X_valid) \ (X_valid' * Y_valid);

    %% Compute robust standard errors with clustering by cohort
    cohort_ids = data.treatment_year(valid_rows);
    unique_cohorts = unique(cohort_ids(~isnan(cohort_ids)));
    num_cohorts = length(unique_cohorts);
    residuals = Y_valid - X_valid * beta;
    bread = inv(X_valid' * X_valid);
    meat = zeros(size(X_valid, 2));
    for c = 1:num_cohorts
        cohort_rows = (cohort_ids == unique_cohorts(c));
        X_cohort = X_valid(cohort_rows, :);
        res_cohort = residuals(cohort_rows);
        meat = meat + (X_cohort' * res_cohort) * (X_cohort' * res_cohort)';
    end
    V_robust = bread * meat * bread;

    % Degrees of freedom adjustment (Stata-style)
    K = size(X_valid, 2); % Number of parameters
    df_adjustment = num_cohorts / (num_cohorts - 1) * (height(X_valid) / (height(X_valid) -K));
    V_robust = V_robust * df_adjustment;

    se_robust = sqrt(diag(V_robust));

    %% Format results into a Stata-like table
    results_table = table(beta, se_robust, beta ./ se_robust, 2 * (1 - tcdf(abs(beta ./ se_robust), num_cohorts - 1)), ...
        beta - 1.96 * se_robust, beta + 1.96 * se_robust, 'RowNames', variable_names', ...
        'VariableNames', ["Coefficient", "StdErr", "tStat", "pValue", "ConfLow(2.5)", "ConfHigh(97.5)"]);
    disp('Wooldridge(2021): Heterogeneous Treatment Effects Estimation Results:');
    results_table

 
end
